@@ -7,10 +7,11 @@ package GPS::gpsd;
 use strict;
 use vars qw($VERSION);
 use IO::Socket;
-use HTTP::Date;
 use Math::Trig;
+use GPS::gpsd::point;
+use GPS::gpsd::satellite;
 
-$VERSION = sprintf("%d.%02d", q{Revision: 0.2} =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q{Revision: 0.5} =~ /(\d+)\.(\d+)/);
 
 sub new {
   my $this = shift;
@@ -26,34 +27,70 @@ sub initialize {
   my %param = @_;
   $self->host($param{'host'} || 'localhost');
   $self->port($param{'port'} || '2947');
-  $self->send($param{'send'} || 'SDPTVAE');
-  #I allow the user to set the send property.  However, SDPTV are required
-  #by built in functions
-}
-
-sub register {
-  my $self = shift();
-  my %param = @_;
-  my $sub=$param{'sub'} || die('Error: sub=>\&{} required');
-  my $filter=$param{'filter'} || \&{1};
-  while (1) {
-    my $data=$self->get();
-    if ($data->{'S'}->[0] > 0) { #if gps fix
-      if (&{$filter}($data)) {   #if true filter
-        &{$sub}($data);
-        sleep 1; 
-      }
+  $self->send($param{'send'} || 'SDOV');
+  unless ($param{'do_not_init'}) { #for testing
+    my $data=$self->retrieve('LKIFCB');
+    foreach (keys %$data) {
+      $self->{$_}=$data->{$_};
     }
   }
 }
 
+sub subscribe {
+  my $self = shift();
+  my %param = @_;
+  my $last=undef();
+  my $handler=$param{'handler'} || \&default_handler;
+  while (1) {
+    my $point=$self->get();
+    if (defined($point) and $point->fix) { #if gps fix
+      my $return=&{$handler}($last, $point);
+      if ($return) {
+        $last=$return;
+      }
+    }
+    sleep 1; 
+  }
+}
+
+sub default_handler {
+  my $p1=shift(); #last true return or undef if first
+  my $p2=shift(); #current fix
+  if (defined($p1)) {
+    if (rand(1) < 0.5) {
+      print "\n",$p1->lat," ",$p1->lon," -> ",$p2->lat," ",$p2->lon,"\n";
+      return $p2;
+    } else {
+      {local $|=1; print ".";}
+      return undef();
+    }
+  } else {
+    print "First: ",$p2->lat," ",$p2->lon,"\n";
+    return $p2;
+  }
+}
+
+sub getsatellitelist {
+  my $self=shift();
+  my $string='Y';
+  my $data=$self->retrieve($string);
+  return GPS::gpsd::satellite->list($data);
+}
+
 sub get {
   my $self=shift();
+  my $string=$self->send();
+  my $data=$self->retrieve($string);
+  return GPS::gpsd::point->new($data);
+}
+
+sub retrieve {
+  my $self=shift();
+  my $string=shift();
   my $sock=$self->open();
   my $data='';
-
   if (defined($sock)) {
-    $sock->send($self->send()) or die("Error: $!");
+    $sock->send($string) or die("Error: $!");
     $sock->recv($data, 256); #Not sure if 256 is good here!
     chomp $data;
     return $self->parse($data);
@@ -79,7 +116,11 @@ sub parse {
   my @line=split(/[,\n\r]/, $line);  
   foreach (@line) {
     if (m/(.*)=(.*)/) {
-      $data{$1}=[split(/ /, $2)];
+      if ($1 eq 'Y') {
+        $data{$1}=[split(/:/, $2)]; #Y is : delimited
+      } else {
+        $data{$1}=[split(/ /, $2)];
+      }
     }
   }
   return \%data;
@@ -108,9 +149,7 @@ sub time {
   my $self=shift();
   my $p1=shift();
   my $p2=shift();
-  my $d1=str2time($p1->{'D'}->[0]);
-  my $d2=str2time($p2->{'D'}->[0]);
-  return abs($d2-$d1);
+  return abs($p2->time - $p1->time);
 }
 
 sub distance {
@@ -118,64 +157,103 @@ sub distance {
   my $self=shift();
   my $p1=shift();
   my $p2=shift();
-  my $x1=$p1->{'P'}->[1];
-  my $y1=$p1->{'P'}->[0];
-  my $x2=$p2->{'P'}->[1];
-  my $y2=$p2->{'P'}->[0];
+  my $y1=$p1->lat;
+  my $x1=$p1->lon;
+  my $y2=$p2->lat;
+  my $x2=$p2->lon;
   return sqrt(($x2-$x1)**2 + ($y2-$y1)**2) * 40075.16 / 360 * 1000;
 }
 
 sub track {
-  #return calculated point of $p1 at time $p2 assuming constant velocity
+  #return calculated point of $p1 in time assuming constant velocity
   my $self=shift();
   my $p1=shift();
-  my $p2=shift();
-  my $x1=$p1->{'P'}->[1];
-  my $y1=$p1->{'P'}->[0];
-  my $heading=$p1->{'T'}->[0];        #degrees from the North
+  my $time=shift();
+  my $x1=$p1->lon;
+  my $y1=$p1->lat;
+  my $heading=$p1->heading;        #degrees from the North
   my $speed=$p1->{'V'}->[0];          #knots; 1knot=1min/hr=1/3600 deg/sec
-  my $time=$self->time($p1,$p2);
   my $x1v=$x1 + sin(deg2rad($heading)) * $speed/3600 * $time;
   my $y1v=$y1 + cos(deg2rad($heading)) * $speed/3600 * $time;
-  my $p1v={%$p1};
-  $p1v->{'P'}=[$y1v, $x1v];
+  my $p1v=GPS::gpsd::point->new($p1);
+  $p1v->{'P'}=[$y1v, $x1v];  #Is there a better OO way to make assignments?
   return $p1v;
 }
 
+sub baud {
+  my $self = shift();
+  return q2u $self->{'B'}->[0];
+}
+
+sub rate {
+  my $self = shift();
+  return q2u $self->{'C'}->[0];
+}
+
+sub device {
+  my $self = shift();
+  return q2u $self->{'F'}->[0];
+}
+
+sub identification {
+  my $self = shift();
+  return q2u $self->{'I'}->[0];
+}
+
+sub id {
+  my $self = shift();
+  return $self->identification;
+}
+
+sub protocol {
+  my $self = shift();
+  return q2u $self->{'L'}->[0];
+}
+
+sub daemon {
+  my $self = shift();
+  return q2u $self->{'L'}->[1];
+}
+
+sub commands {
+  my $self = shift();
+  return q2u $self->{'L'}->[2];
+}
+
+sub q2u {
+  my $a=shift();
+  return $a eq '?' ? undef() : $a;
+}
+
 1;
 __END__
 
 =head1 NAME
 
-  GPS::gpsd is a module that provides a perl interface to the gpsd daemon. gpsd is an open source gps deamon from http://gpsd.berlios.de/.
+GPS::gpsd is a module that provides a perl interface to the gpsd
+daemon. gpsd is an open source gps deamon from http://gpsd.berlios.de/.
 
 =head1 SYNOPSIS
 
-  use GPS::gpsd;
-  $gps = new GPS::gpsd(  host    => 'localhost',
-	  		 port      => 2947
-                );
-  my $fix=$gps->get();
+use GPS::gpsd;
+$gps=new GPS::gpsd();
+my $point=$gps->get();
+print $point->lat, " ", $point->lon, "\n";
 
-  or
-
-  use GPS::gpsd;
-  $gps->register(sub=>\&gps_handler);
-  sub gps_handler {
-    my $data=shift();
-    print "Lat:". $data->{'P'}->[0]. " Lon:". $data->{'P'}->[1]. "\n";
-  }
+or
 
+use GPS::gpsd;
+$gps=new GPS::gpsd();
+$gps->subscribe();
 
 =head1 DESCRIPTION
 
-  GPS::gpsd provides a very simple interface to gpsd daemon in perl scripts.
+GPS::gpsd provides a very simple interface to gpsd daemon in perl scripts.
  
-  For example the method get() returns a hash reference like
-                     {S=>[?|0|1|2],
-                      P=>[lat,lon]}
+For example the method get() returns a hash reference like
+        {S=>[?|0|1|2], P=>[lat,lon]}
 
-  Unfortunately you'll still need to know the single character protocol of the gpsd daemon.
+Fortunately, there are various methods that hide this hash from the user.
 
 =over
 
@@ -183,8 +261,6 @@ __END__
 
 =head1 KNOWN LIMITATIONS
 
-  The subroutine registration could use some more work.  I'd like to add a min time, max time, position tolerance and tracking tolerance capabilities.
-
 =head1 BUGS
 
   No known bugs.
@@ -196,8 +272,7 @@ use strict;
 use lib './';
 use GPS::gpsd;
 
-my $gps=GPS::gpsd->new(host=>'192.168.33.130',
-                       port=>2947);
+my $gps=GPS::gpsd->new();
 my $data=$gps->get();
 my %fix=('?'=>"Error", 0=>"No Fix", 1=>"Fix", 2=>"DGPS-Corrected Fix");
 print "gpsd.pm Version:", $gps->VERSION, "\n";
@@ -206,11 +281,12 @@ print "Fix:", $data->{'S'}->[0], "=", $fix{$data->{'S'}->[0]}, "\n";
 print "Lat:", $data->{'P'}->[0], " Lon:", $data->{'P'}->[1], "\n";
 print "Host:", $gps->host, " Port:", $gps->port, "\n";
 
-$gps->register(sub=>\&gps_handler);
+$gps->subscribe(handler=>\&gps_handler);
 
 sub gps_handler {
-  my $data=shift();
-  print join " ", "Fix", $data->{'S'}->[0], $data->{'P'}->[0], $data->{'P'}->[1], "\n";
+  my $point=shift();
+  print join " ", "Fix", $point->{'S'}->[0], $point->{'P'}->[0], $point->{'P'}->[1], "\n";
+  return $point
 }
 
 =head1 AUTHOR
